(function (root, factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define(['api/FundingAccountApi', 'model/FundingAccountToken', 'model/ConfirmAddFundAccTokenRequest'], factory);
    } else if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('./FundingAccountApi'), require('../model/FundingAccountToken'), require('../model/ConfirmAddFundAccTokenRequest'));
    } else {
        // Browser globals (root is window)
        if (!root.OrbipayCheckoutApIs) {
            root.OrbipayCheckoutApIs = {};
        }
        root.OrbipayCheckoutApIs.FundingAccountTokenApi = factory(root.OrbipayCheckoutApIs.FundingAccountApi, root.OrbipayCheckoutApIs.FundingAccountToken, root.OrbipayCheckoutApIs.ConfirmAddFundAccTokenRequest);
    }
}(this, function (FundingAccountApi, FundingAccountToken, ConfirmAddFundAccTokenRequest) {
    'use strict';

    var exports = function (idAccount) {

        this.fundAccount = new FundingAccountApi();
        this.confirmAddFundAccTokenRequest = new ConfirmAddFundAccTokenRequest();
        this.fundAccountToken = new FundingAccountToken();
        this.token = null;
        this.digiSign = null;
        this.customFields = null;

        this.forPartner = function(clientId, signatureKey){
            this.fundAccount.apiClient.clientId = clientId;
            this.fundAccount.apiClient.signatureKey = signatureKey;
            return this;
        }

        // this.forClient = function(clientId, signatureKey){
        //     this.fundAccount.apiClient.clientId = clientId;
        //     this.fundAccount.apiClient.signatureKey = signatureKey;
        //     return this;
        // }

        this.forClient = function(clientId, signatureKey, clientApiKey){
            if(typeof clientApiKey !== 'undefined'){
                this.fundAccount.apiClient.clientId = clientId;
                this.fundAccount.apiClient.signatureKey = signatureKey;
                this.fundAccount.apiClient.clientApiKey = clientApiKey;
                return this;
            }else {
                this.fundAccount.apiClient.clientId = clientId;
                this.fundAccount.apiClient.signatureKey = signatureKey;
                return this;
            }

        }

        this.withToken = function(token, digiSign){
            if(typeof token === 'string' || token instanceof String){
                this.fundAccountToken.token = token;
            }else{
                this.fundAccountToken = token;
            }
            this.digiSign = digiSign;
            return this;
        }

        this.add = function(callback,liveMode){
            if(typeof liveMode !== 'undefined'){
                console.log("in add function with livemode set");
                this.fundAccount.apiClient.liveMode = liveMode;
            } else{
                console.log("in add function without liveMode");
            }
            var decryptedToken = this.fundAccount.apiClient.decryptStringWithRsaPrivateKey(this.fundAccountToken.token);
            if(this.fundAccount.apiClient.verifyDigiSignWithPublicKey(this.digiSign, decryptedToken)){
                this.fundAccountToken.token = this.fundAccount.apiClient.encryptStringWithRsaPublicKey(decryptedToken);
                this.digiSign = this.fundAccount.apiClient.digiSignWithPrivateKey(decryptedToken);
                this.confirmAddFundAccTokenRequest.funding_account_token = this.fundAccountToken;
            } else {
                console.log("Validation failed !!!!!!!!!")
            }

            this.fundAccount.confirmAddFundingAccount(this.fundAccount.apiClient.clientId,  this.digiSign, this.confirmAddFundAccTokenRequest, callback, false);
        }

        this.update = function(callback,liveMode){
            if(typeof liveMode !== 'undefined'){
                console.log("in update function with livemode set");
                this.fundAccount.apiClient.liveMode = liveMode;
            } else{
                console.log("in update function without liveMode");
            }
            var decryptedToken = this.fundAccount.apiClient.decryptStringWithRsaPrivateKey(this.fundAccountToken.token);
            if(this.fundAccount.apiClient.verifyDigiSignWithPublicKey(this.digiSign, decryptedToken)){
                this.fundAccountToken.token = this.fundAccount.apiClient.encryptStringWithRsaPublicKey(decryptedToken);
                this.digiSign = this.fundAccount.apiClient.digiSignWithPrivateKey(decryptedToken);
                this.confirmAddFundAccTokenRequest.funding_account_token = this.fundAccountToken;
            } else {
                console.log("Validation failed !!!!!!!!!")
            }

            this.fundAccount.confirmAddFundingAccount(this.fundAccount.apiClient.clientId,  this.digiSign, this.confirmAddFundAccTokenRequest, callback, true, idAccount);
        }
    };

    return exports;
}));
